import { Nested, TypeSaveProperty } from 'dotup-ts-types';
import generator from 'yeoman-generator';
import { Project } from './project/Project';
import { ProjectInfo } from './project/ProjectInfo';
import { IStepQuestion } from './questions/IStepQuestion';
import { Question } from './questions/Question';
import { SharedOptions } from './SharedOptions';
import { GeneratorOptions } from './Types';
export declare abstract class BaseGenerator<TStep extends string> extends generator {
    static counter: number;
    static sharedOptions: SharedOptions<string>;
    readonly sharedOptions: SharedOptions<TStep>;
    readonly projectInfo: ProjectInfo;
    private readonly doNotEjsReplace;
    private readonly skippedTemplateFiles;
    private readonly generatorName;
    skipQuestions: boolean;
    skipGenerator: boolean;
    projectFiles: Project;
    conflictedProjectFiles: Project;
    answers: TypeSaveProperty<Nested<TStep, string>>;
    questions: IStepQuestion<TStep>[];
    currentStep: TStep;
    constructor(args: string | string[], options: GeneratorOptions<TStep>);
    compose(generator: string, passThroughAnswers?: boolean, options?: any): void;
    addSkipEjsReplacement(targetPath: string): void;
    addSkippedTemplateFiles(targetPath: string): void;
    isAnswered(): boolean;
    getQuestion(name: TStep): IStepQuestion<TStep>;
    tryGetAnswer(questionName: TStep): string;
    mergeAnswers(): void;
    setRootPath(rootPath?: string): void;
    registerMethod(self: BaseGenerator<TStep>): void;
    addQuestion(question: Question<TStep>): void;
    addStepQuestion(stepName: TStep, question: IStepQuestion<TStep>): void;
    getDefaultProjectName(): string;
    destinationIsProjectFolder(projectName: string): boolean;
    writeOptionsToAnswers(propertyDescriptor: any): any;
    validateString(value: string): boolean;
    loadTemplateFiles(): void;
    logGreen(message: string): void;
    logRed(message: string): void;
    logBlue(message: string): void;
    logYellow(message: string): void;
    /**
     *
     * OVERRIDES
     *
     */
    destinationRoot(rootPath?: string): string;
    composeWith(namespace: any, options: {
        [name: string]: any;
    }, settings?: {
        local: string;
        link: 'weak' | 'strong';
    }): this;
    /**
     *
     * RUN LOOP
     *
     */
    /**
     * Your initialization methods(checking current project state, getting configs, etc)
     */
    initializing(): Promise<void>;
    /**
     * Where you prompt users for options(where you’d call this.prompt())
     */
    prompting(): Promise<void>;
    /**
     * Saving configurations and configure the project(creating.editorconfig files and other metadata files)
     */
    configuring(): Promise<void>;
    copyTemplateFiles(): Promise<void>;
    /**
     * Where you write the generator specific files(routes, controllers, etc)
     */
    default(): Promise<void>;
    writing(): Promise<void>;
    /**
     * Where conflicts are handled(used internally)
     */
    /**
     * Where installations are run(npm, bower)
     */
    install(): Promise<void>;
    /**
     * Called last, cleanup, say good bye, etc
     */
    end(): Promise<void>;
}
