"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const fs = __importStar(require("fs"));
const _ = __importStar(require("lodash"));
const npm_registry_api_1 = require("npm-registry-api");
const path = __importStar(require("path"));
const yeoman_generator_1 = __importDefault(require("yeoman-generator"));
const Project_1 = require("./project/Project");
const ProjectInfo_1 = require("./project/ProjectInfo");
const ProjectPathAnalyser_1 = require("./project/ProjectPathAnalyser");
const project_1 = require("./project");
class BaseGenerator extends yeoman_generator_1.default {
    constructor(args, options) {
        super(args, options);
        this.doNotEjsReplace = [];
        this.skippedTemplateFiles = [];
        this.skipQuestions = false;
        this.skipGenerator = false;
        this.answers = {};
        this.questions = [];
        BaseGenerator.counter += 1;
        BaseGenerator.sharedOptions = options['sharedOptions']; // (<IProperty>options)['sharedOptions'];
        if (this.sharedOptions !== undefined) {
            _.merge(options, this.sharedOptions.values);
        }
        if (options.skippedTemplateFiles !== undefined) {
            options.skippedTemplateFiles.forEach(f => {
                this.addSkippedTemplateFiles(f);
            });
        }
        this.generatorName = this.constructor.name;
        this.projectInfo = new ProjectInfo_1.ProjectInfo();
        this.setRootPath();
    }
    get sharedOptions() {
        return BaseGenerator.sharedOptions;
    }
    compose(generator, passThroughAnswers = true, options) {
        const optArgs = options || {};
        if (passThroughAnswers) {
            _.merge(optArgs, this.answers);
        }
        optArgs['sharedOptions'] = this.sharedOptions;
        const filePath = require.resolve(generator, {
            paths: [
                path.join(this.sourceRoot(), '..')
            ]
        });
        this.composeWith(filePath, optArgs);
    }
    // trySubscribeSharedOption(questionName: TStep | string): void {
    //   if (this.sharedOptions !== undefined) {
    //     this.sharedOptions.subscribe(this, questionName);
    //   }
    // }
    addSkipEjsReplacement(targetPath) {
        this.doNotEjsReplace.push(targetPath);
    }
    addSkippedTemplateFiles(targetPath) {
        const tmp = path.normalize(targetPath);
        this.skippedTemplateFiles.push(tmp);
    }
    isAnswered() {
        const required = this.questions.filter(item => item.isRequired === true);
        // tslint:disable-next-line: no-any
        return required.every(item => this.answers[item.name] !== undefined);
    }
    getQuestion(name) {
        return this.questions.find(item => item.name === name);
    }
    tryGetAnswer(questionName) {
        if (this.answers[questionName] !== undefined) {
            return this.answers[questionName];
        }
        else if (this.sharedOptions === undefined) {
            return undefined;
        }
        else {
            return this.sharedOptions.getAnswer(questionName);
        }
    }
    mergeAnswers() {
        if (this.sharedOptions === undefined) {
            return;
        }
        _.merge(this.answers, this.sharedOptions.values);
    }
    setRootPath(rootPath) {
        if (this.sharedOptions === undefined || this.sharedOptions.rootPath === undefined) {
            return;
        }
        // We're in the wrong folder, try to set root
        if (this.destinationPath() !== this.sharedOptions.rootPath) {
            this.sourceRoot(this.sharedOptions.rootPath);
        }
        // If the destination path still points to another directory, a yo file is in parent folder.
        if (this.destinationPath() !== this.sharedOptions.rootPath) {
            this.logRed(`${this.generatorName}: Project target path is ${this.destinationPath()}`);
            this.logRed(`You've to delete the yo file to continue: ${this.destinationPath('.yo-rc.json')}`);
            throw new Error(`You've to delete the yo file to continue: ${this.destinationPath('.yo-rc.json')}`);
        }
        else {
            this.logGreen(`${this.generatorName}: Project target path is ${this.destinationPath()}`);
        }
    }
    registerMethod(self) {
        const methods = [
            'prompting', 'configuring', 'default', 'writing'
        ];
        methods.forEach(method => {
            // tslint:disable-next-line: no-unsafe-any
            self.constructor.prototype[method] = this[method];
        });
    }
    addQuestion(question) {
        this.addStepQuestion(question.name, question);
    }
    addStepQuestion(stepName, question) {
        // Avoid registering twice
        if (this.getQuestion(stepName) !== undefined) {
            throw new Error(`Question '${stepName}' already configured.`);
        }
        // If the name isn't set..
        if (question.name === undefined) {
            question.name = stepName;
        }
        // Build generator options
        // if (question.isOption) {
        //   this.option(question.name, {
        //     type: question.optionType || String,
        //     description: typeof question.message === 'function' ? '' : question.message // 'Name of the repository'
        //   });
        // }
        // With the first question
        if (this.currentStep === undefined) {
            this.currentStep = stepName;
        }
        else {
            // Set next question
            const keys = Object.keys(this.questions);
            // tslint:disable-next-line: no-any
            const prevQuestion = this.questions[keys[keys.length - 1]];
            if (prevQuestion !== undefined && prevQuestion.nextQuestion === undefined) {
                prevQuestion.nextQuestion = stepName;
            }
        }
        // Add to questions
        // this.questions[stepName] = question;
        this.questions.push(question);
    }
    getDefaultProjectName() {
        const opt = this.options;
        if (opt.projectName) {
            return _.kebabCase(opt.projectName);
        }
        else {
            return _.kebabCase(this.appname);
        }
    }
    destinationIsProjectFolder(projectName) {
        const root = path.basename(this.destinationPath());
        if (root.toLowerCase() === projectName.toLowerCase()) {
            return true;
        }
        else {
            return false;
        }
    }
    // tslint:disable-next-line: no-any
    writeOptionsToAnswers(propertyDescriptor) {
        const keys = Object
            // tslint:disable-next-line: no-unsafe-any
            .keys(propertyDescriptor)
            .map(x => x);
        const opt = this.options;
        keys.forEach(key => {
            if (opt[key] !== undefined) {
                this.answers[key] = opt[key];
                if (BaseGenerator.sharedOptions) {
                    BaseGenerator.sharedOptions.setAnswer(key, opt[key]);
                }
            }
        });
    }
    validateString(value) {
        if (value !== undefined && value.length > 0) {
            return true;
        }
        else {
            this.logRed(`${this.getQuestion(this.currentStep).message} is required.`);
            return false;
        }
    }
    loadTemplateFiles() {
        if (this.skipGenerator)
            return;
        this.logBlue(`Analyse template files. (${this.generatorName})`);
        const x = new ProjectPathAnalyser_1.ProjectPathAnalyser((...args) => this.templatePath(...args));
        this.projectFiles = x.getProjectFiles(this.projectInfo);
    }
    logGreen(message) {
        this.log(chalk_1.default.green(message));
    }
    logRed(message) {
        this.log(chalk_1.default.red(message));
    }
    logBlue(message) {
        this.log(chalk_1.default.blue(message));
    }
    logYellow(message) {
        this.log(chalk_1.default.yellow(message));
    }
    /**
     *
     * OVERRIDES
     *
     */
    destinationRoot(rootPath) {
        if (this.sharedOptions !== undefined) {
            this.sharedOptions.setRootPath(rootPath);
        }
        return super.destinationRoot(rootPath);
    }
    composeWith(namespace, options, settings) {
        const optArgs = options || {};
        _.merge(optArgs, this.answers);
        optArgs['sharedOptions'] = this.sharedOptions;
        return super.composeWith(namespace, options, settings);
    }
    /**
     *
     * RUN LOOP
     *
     */
    /**
     * Your initialization methods(checking current project state, getting configs, etc)
     */
    async initializing() { }
    /**
     * Where you prompt users for options(where you’d call this.prompt())
     */
    async prompting() {
        if (this.skipGenerator)
            return;
        if (this.skipQuestions || this.questions.length < 1) {
            return;
        }
        // No entry point
        if (this.currentStep === undefined) {
            throw new Error('Initial step not set');
        }
        do {
            // Do we have user input?
            let hasInput = false;
            // Get current question
            const question = this.getQuestion(this.currentStep);
            // Set name to avoid writing the name twice on the definition
            question.name = this.currentStep;
            // Should ask?
            let ask = true;
            if (question.When !== undefined) {
                ask = await question.When(this.answers);
            }
            if (ask) {
                // Prompt
                const answer = await this.prompt(question);
                // Store answer
                if (answer[this.currentStep] !== undefined) {
                    hasInput = true;
                    this.answers[this.currentStep] = answer[this.currentStep];
                    if (BaseGenerator.sharedOptions) {
                        BaseGenerator.sharedOptions.setAnswer(this.currentStep, answer[this.currentStep]);
                    }
                }
            }
            // Accept answer callback configured?
            if (hasInput && question.acceptAnswer !== undefined) {
                const accepted = await this
                    .getQuestion(this.currentStep)
                    .acceptAnswer(this.answers[this.currentStep]);
                // Should we ask again same step?
                if (accepted === true) {
                    // Maybe answer changed in callback
                    if (BaseGenerator.sharedOptions) {
                        BaseGenerator.sharedOptions.setAnswer(this.currentStep, this.answers[this.currentStep]);
                    }
                    // Set next step
                    this.currentStep = question.nextQuestion;
                }
            }
            else {
                // Set next step
                this.currentStep = question.nextQuestion;
            }
        } while (this.currentStep !== undefined);
    }
    /**
     * Saving configurations and configure the project(creating.editorconfig files and other metadata files)
     */
    async configuring() {
        if (this.skipGenerator)
            return;
        // tslint:disable-next-line: no-backbone-get-set-outside-model
        // this.config.set('answers', this.answers);
        // this.config.save();
    }
    async copyTemplateFiles() {
        if (this.skipGenerator)
            return;
        this.mergeAnswers();
        this.conflictedProjectFiles = new Project_1.Project(this.projectInfo);
        this.projectFiles.templateFiles.forEach(file => {
            // Get the file extension
            let ext = path.extname(file.targetPath);
            // Remove the extension on renamable files
            if (file.typ === project_1.TemplateType.removeExtension) {
                ext = path.extname(file.targetPath);
            }
            if (ext === '') {
                ext = path.basename(file.targetPath);
            }
            // Skip this file?
            if (this.skippedTemplateFiles.includes(file.targetPath)) {
                return;
            }
            if (this.fs.exists(this.destinationPath(file.targetPath))) {
                switch (ext) {
                    case '.ts':
                        throw new Error(`Resolving conflicted ${ext} files not implemented. ${file}`);
                    case '.json':
                        const newJsonContent = {};
                        const existingJsonFileContent = this.fs.read(file.targetPath, 'utf-8');
                        const existingJsonContent = JSON.parse(existingJsonFileContent);
                        const addJsonFileContent = fs.readFileSync(file.filePath, 'utf-8');
                        const addJsonContent = JSON.parse(addJsonFileContent);
                        _.mergeWith(newJsonContent, addJsonContent, (objValue, srcValue) => {
                            if (_.isArray(objValue)) {
                                return objValue.concat(srcValue);
                            }
                        });
                        _.mergeWith(newJsonContent, existingJsonContent, (objValue, srcValue) => {
                            if (_.isArray(objValue)) {
                                return objValue.concat(srcValue);
                            }
                        });
                        // tslint:disable-next-line: no-unsafe-any
                        // this.fs.extendJSON(this.destinationPath(file.targetPath), addJsonContent);
                        this.fs.write(this.destinationPath(file.targetPath), JSON.stringify(newJsonContent, undefined, 2));
                        if (!this.doNotEjsReplace.includes(file.targetPath)) {
                            this.fs.copyTpl(this.destinationPath(file.targetPath), this.destinationPath(file.targetPath), this.answers);
                        }
                        break;
                    case '.yml':
                    case '.txt':
                    case '.md':
                    case '.gitignore':
                    case '.npmignore':
                        const newGitContent = fs.readFileSync(file.filePath, 'utf-8');
                        const gitContent = this.fs.read(this.destinationPath(file.targetPath), 'utf-8');
                        const newFileContent = `${gitContent}\n\n# ${this.generatorName} related:\n${newGitContent}`;
                        this.fs.write(this.destinationPath(file.targetPath), newFileContent);
                        if (!this.doNotEjsReplace.includes(file.targetPath)) {
                            this.fs.copyTpl(this.destinationPath(file.targetPath), this.destinationPath(file.targetPath), this.answers);
                        }
                        break;
                    default:
                        this.conflictedProjectFiles.templateFiles.push(file);
                        throw new Error(`Could not resolve conflicted ${ext} files. ${file}`);
                }
            }
            else {
                switch (ext) {
                    case '.yml':
                    case '.txt':
                    case '.js':
                    case '.md':
                    case '.json':
                    case '.gitignore':
                    case '.npmignore':
                        if (this.doNotEjsReplace.includes(file.targetPath)) {
                            this.fs.copy(file.filePath, this.destinationPath(file.targetPath));
                        }
                        else {
                            this.fs.copyTpl(file.filePath, this.destinationPath(file.targetPath), this.answers);
                        }
                        break;
                    default:
                        this.fs.copy(file.filePath, this.destinationPath(file.targetPath));
                }
            }
        });
        const npm = new npm_registry_api_1.NpmApi();
        const packegeJson = this.fs.readJSON(this.destinationPath('package.json'));
        if (packegeJson !== undefined) {
            await npm.updateDependencies(packegeJson);
            this.fs.writeJSON('package.json', packegeJson);
        }
    }
    /**
     * Where you write the generator specific files(routes, controllers, etc)
     */
    // tslint:disable-next-line: no-reserved-keywords
    async default() {
        if (this.skipGenerator)
            return;
        this.setRootPath();
        this.loadTemplateFiles();
    }
    async writing() {
        if (this.skipGenerator)
            return;
        await this.copyTemplateFiles();
    }
    /**
     * Where conflicts are handled(used internally)
     */
    // abstract async conflicts(): Promise<void>;
    // async resolveConflicts(): Promise<void> {
    //   const conflicted = this.conflictedProjectFiles.templateFiles;
    //   conflicted.forEach(file => {
    //     const ext = path.extname(file.filePath);
    //     switch (ext) {
    //       case '.ts':
    //         throw new Error(`Resolving conflicted ${ext} files not implemented.`);
    //       case '.json':
    //         const fileContent = fs.readFileSync(file.filePath, 'utf-8');
    //         const addJsonContent = JSON.parse(fileContent);
    //         this.fs.extendJSON(this.destinationPath(file.targetPath), addJsonContent);
    //         break;
    //       default:
    //         throw new Error(`Could not resolve conflicted ${ext} files.`);
    //     }
    //   });
    // }
    /**
     * Where installations are run(npm, bower)
     */
    async install() { }
    /**
     * Called last, cleanup, say good bye, etc
     */
    async end() { }
}
BaseGenerator.counter = 0;
exports.BaseGenerator = BaseGenerator;

//# sourceMappingURL=BaseGenerator.js.map
